/*
 *  TOPPERS/SSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2007 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2010 by Meika Sugimoto
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 */

/*
 * シリアルドライバ（STM32F0-Discovery用）
 */

#include <kernel.h>
#include <sil.h>
#include "target_intr.h"
#include "target_serial.h"
#include "target_syssvc.h"
#include "stm32f0xx.h"

/*
 * レジスタ設定値
 */
#define PORT2SIOPID(x)	((x) + 1)
#define INDEX_PORT(x)	((x) - 1)
#define GET_SIOPCB(x)	(&siopcb_table[INDEX_PORT(x)])
#define CR1_UE			(0x0001)
#define CR1_TXEIE		(0x0080)
#define CR1_RXNEIE		(0x0020)
#define CR1_TE			(0x0008)
#define CR1_RE			(0x0004)


/*
 * シリアルI/Oポート管理ブロックエリア
 */
SIOPCB siopcb_table[TNUM_PORT];

static USART_TypeDef *sioreg_table[TNUM_PORT] = {
	USART1
#if (TNUM_PORT >= 2)
	USART2
#endif
};

Inline bool_t  sio_putready(SIOPCB* siopcb)
{
	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;
	return USART_GetFlagStatus( pUART, USART_FLAG_TXE) != RESET ;
}

Inline bool_t sio_getready(SIOPCB* siopcb)
{
	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;
	return USART_GetFlagStatus( pUART, USART_FLAG_RXNE) != RESET;
}

/*
 *  低レベル出力の初期化
 */

void target_low_output_init(ID siopid)
{
	USART_TypeDef *pUART = sioreg_table[INDEX_PORT(siopid)];

	/* 初期化済みの場合はすぐにリターン */
	if ( ( pUART->CR1 &  (CR1_RE | CR1_TE | CR1_UE) ) == (CR1_RE | CR1_TE | CR1_UE))
	{
		return ;
	}
	/* USARTの無効化 */
	USART_Cmd( pUART, DISABLE);

	USART_InitTypeDef USART_InitStructure;

	USART_InitStructure.USART_BaudRate = BPS_SETTING;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init( pUART, &USART_InitStructure);  

	//NVIC_Configuration();

	USART_Cmd( pUART, ENABLE);  
}

/*
 *  ターゲットのシリアル初期化
 */
void target_usart_init(ID siopid)
{
	USART_TypeDef *pUART = sioreg_table[INDEX_PORT(siopid)];
	
	target_low_output_init(siopid);
	
	/* エラー割込みの有効化 */
	USART_ITConfig( pUART, USART_IT_ERR, ENABLE );
	
	/* 割込み許可 */
	(void)ena_int(INTNO_SIO);
}

/*
 *  ターゲットのシリアル終了
 */
void target_usart_term(ID siopid)
{
	USART_TypeDef *pUART = sioreg_table[INDEX_PORT(siopid)];
	
	/* USARTの無効化 */
	USART_Cmd( pUART, DISABLE );
}

/*
 *  SIO初期化
 */
void sio_initialize(intptr_t exinf)
{
	int i;

	for (i = 0; i < TNUM_PORT; i++) {
		siopcb_table[i].port = i;
		siopcb_table[i].reg = (uint32_t)sioreg_table[i];
		siopcb_table[i].exinf = 0;
	}
}

/*
 *  シリアルオープン
 */
SIOPCB *sio_opn_por(ID siopid, intptr_t exinf)
{
	SIOPCB* siopcb;

	if (siopid > TNUM_PORT) {
		return NULL;
	}
	
	siopcb = GET_SIOPCB(siopid);
	siopcb->exinf = exinf;

	target_usart_init(siopid);

	return siopcb;
}

/*
 *  シリアルクローズ
 */
void sio_cls_por(SIOPCB *p_siopcb)
{
	target_usart_term(PORT2SIOPID(p_siopcb->port));
}

/*
 *  割込みハンドラ
 */
void sio_isr(intptr_t exinf)
{
	SIOPCB* siopcb = GET_SIOPCB(exinf);

	if (sio_putready(siopcb)) {
		sio_irdy_snd(siopcb->exinf);
	}
	if (sio_getready(siopcb)) {
		sio_irdy_rcv(siopcb->exinf);
	}
}

/*
 *  1文字送信
 */
bool_t sio_snd_chr(SIOPCB *siopcb, char_t c)
{

	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;
	if (sio_putready(siopcb)) {
		USART_SendData( pUART,  c );

		return true;
	}
	return false;

}

/*
 *  1文字受信
 */
int_t sio_rcv_chr(SIOPCB *siopcb)
{
	int_t c = -1;
	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;

	if (sio_getready(siopcb)) {
		c = USART_ReceiveData( pUART ) & 0xFF;
	}

	return c;
}

/*
 *  コールバックの許可
 */
void sio_ena_cbr(SIOPCB *siopcb, uint_t cbrtn)
{
	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;
	switch (cbrtn) {
	case SIO_RDY_SND:
		USART_ITConfig( pUART, USART_IT_TXE , ENABLE );
		break;
	case SIO_RDY_RCV:
		USART_ITConfig( pUART, USART_IT_RXNE , ENABLE );
		break;
	default:
		break;
	}
}

/* 
 *  コールバックの禁止
 */
void sio_dis_cbr(SIOPCB *siopcb, uint_t cbrtn)
{
	USART_TypeDef *pUART = ( USART_TypeDef *)siopcb->reg;
	switch (cbrtn) {
	case SIO_RDY_SND:
		USART_ITConfig( pUART, USART_IT_TXE , DISABLE );
		break;
	case SIO_RDY_RCV:
		USART_ITConfig( pUART, USART_IT_RXNE , DISABLE );
		break;
	default:
		break;
	}
}

/*
 *  1文字出力（ポーリングでの出力）
 */
void sio_pol_snd_chr(char_t c, ID siopid)
{
	USART_TypeDef *pUART = sioreg_table[INDEX_PORT(siopid)];
	USART_SendData( pUART, c );
	while (  USART_GetFlagStatus( pUART, USART_FLAG_TXE )  == RESET ) { /* wait */ }
}

